/************************************************************************
 ***         Beispielprogramm fr die PC-Steckkarte AD1199            ***
 ***         ERMA Electronic GmbH, 78194 Immendingen                  ***
 ************************************************************************/

#include <stdio.h>                       /* Prototypen fr Ein-Ausgabe */
#include <stdlib.h>
#include <conio.h>
#include <dos.h>

/*** Prozedurprototypen (Global) ***/
void clear_screen(void);                /* Prototyp Bildschirm lschen  */
void write_position(int x,int y,char *text,int attr);/* Prototyp fr positionierte Ausgabe */
int read_datefile(void);                /* Prototyp fr Lesen Datenfile */
int write_datefile(void);               /* Prototyp fr Schreiben Datenfile */
float read_board(unsigned char channel);/* Prototyp fr Lesen der Karte */
void main_menu(void);                   /* Prototyp fr Hauptmenu       */
void calibrate_menu(void);              /* Prototyp fr Eichmenu        */
void show_menu(void);                   /* Prototyp fr Anzeigemenu     */
void show(void);                        /* Prototyp fr Anzeigeablauf   */
void calibrate(void);                   /* Prototyp fr Eichablauf      */
void output_menu(void);                 /* Prototyp fr Ausgnge        */
void input_menu(void);                  /* Prototyp fr Eingnge        */

/*** Variablendeklaration (Global) ***/
unsigned address=0x100;                 /* Pointer mit Portadresse      */
struct data{                            /* Struktur fr Eichwerte       */
	     float y1[8],               /* Variablen fr Nullwert       */
		   y2[8],               /* Variablen fr Endwert        */
		   x1[8],               /* Variablen fr Nullwert-Karte */
		   x2[8],               /* Variablen fr Endwert-Karte  */
		   s[8],                /* Variablen fr Steigung Abgleichgerade */
		   n[8];                /* Variablen fr Nullpunktversatz */
	   }date;                       /* Variablen fr Abgleichwerte  */
FILE *date_file;                        /* Filestruktur fr Datenfile   */
char file_name[]="PT100.DAT";           /* Name vom Datenfile           */
int channel=0,output=48;                /*globale Hilfsvariablen        */
int ferr,key;

/************************************************************************
 ***                       Hauptprogramm                              ***
 ************************************************************************/

int main(void)                          /* Rckgabewert des Programms : */
{                                       /* 0=OK, -1=Fehler              */
  int i,flag;                           /* lokale Hilfsvariable         */

  for(i=0;i<=7;i++)
    {
      date.y1[i]=0;
      date.y2[i]=0;
      date.x1[i]=0;
      date.x2[i]=0;
    }
  clear_screen();                       /* Bildschirm lschen           */
  _setcursortype(_NOCURSOR);            /* Cursor aus                   */
  ferr=read_datefile();                 /* Abgleichdatei laden          */
  if (ferr==0)                          /* Wenn laden erfolgreich       */
      {
	main_menu();                    /* Hauptmenu darstellen         */
	key=0;                          /* Variable fr Taste initialisieren */
	do                              /* in Schleife verharren        */
	  {                             /* bis Taste F1,F2 oder F10 gedrckt */
	  i=kbhit();
	  if (i==-1)                    /* Ist Taste gedrckt ?         */
	    {                           /* Wenn ja, dann                */
	      key=getch();              /* ASCII-Wert lesen             */
	      if (key==0)
		key=getch();            /* Scan-Code holen              */
	      switch (key)
		{
		  case 59:{             /* Bei F1 : Anzeigen            */
			    show();
			    main_menu();
			    key=0;
			    break;
			  }
		  case 60:{             /* Bei F2 : Eichen              */
			    calibrate();
			    main_menu();
			    key=0;
			    break;
			  }
		  case 61:{             /* Bei F3 : Digitale Ausgnge   */
			    output_menu();
			    main_menu();
			    key=0;
			    break;
			  }
		  case 62:{             /* Bei F4 : Digitale Eingnge   */
			    input_menu();
			    main_menu();
			    key=0;
			    break;
			  }
		}
	    }
	  }
	while (key!=68);                /* Wenn F10, dann ende          */

	ferr=write_datefile();          /* Abgleichdaten speichern      */
	if (ferr==0)                    /* Wenn speichern erfolgreich   */
	    flag=0;                     /* Rckgabewert = 0             */
	  else
	    flag=-1;                    /* sonst Rckgabewert = -1      */
      }
    else                                /* Wenn laden nicht erfolgreich */
      flag=-1;                          /* Rckgabewert = -1            */
  write_position(1,1," ",0);
  textattr(0x07);
  clrscr();
  _setcursortype(_NORMALCURSOR);        /* Standardcursor an            */
  return flag;
}

/************************************************************************
 ***                Unterprogramme fr AD 1199                        ***
 ************************************************************************/

float read_board(unsigned char channel) /* Gibt eingelesenen Wert zurck */
{
  unsigned char i;                      /* lokale Hilfsvariable         */
  unsigned char date1,date2;            /* Variablen fr digitale Werte */
  int out_loop=10;                       /* Variable fr Meschleife     */
					/* Wird bentigt, da umschalten */
					/* auf anderen Kanal erst nach  */
					/* Beendigung einer Meung erfolgt */
  float wert,wertz=0;                         /* Variable fr rechnerischen Wert */

  outportb(address,(channel | output));
  delay(20);
  for (i=1;i<=out_loop;i++)             /* Meschleife                  */
    {
      outport(address,(channel | output)); /* Ausgabe des gewhlten Kanals */
					/* Warteschleife bis Wert       */
  delay(1);
      do
      {                                 /* gewandelt                    */
	date1=inportb(address+1);       /* Lesen des Kontrollbits       */
      }  while ((date1 & 0x10)!=0x10);
      date1=inportb(address);            /* Lesen Wert1                  */
      date2=(inportb(address+1) & 0x0F); /* Lesen Wert2 (nur 4 Bits )  */
      wert=date2*256+date1;              /* Umrechnen auf Realzahl       */
      wertz+=wert;
    }
    wert=wertz/out_loop;
  return wert;                          /* Wert wird zurckgegeben      */
}

/************************************************************************
 ***                Unterprogramme zur Dateibehandlung                ***
 ************************************************************************/

int read_datefile(void)                 /* Rckgabewert der Funktion    */
{                                       /* 0=OK, -1=Fehler              */
  /* Bildschirmmeldungen */
  char read_error[]=" Fehler beim Lesen der Abgleichdaten ! ";
  char open_error[]=" Fehler beim ffnen der Abgleichdatei ! ";
  char text1[]=" Beenden des Programms mit beliebiger Taste ";
  char wert[30];
  int i,fehler,flag;                      /* lokale Hilfsvariablen        */

  flag=0;
  date_file=fopen(file_name,"rb");      /* Datei zum Lesen ffnen       */
  if (date_file==NULL)                  /* Wenn Fehler aufgetreten      */
      {                                 /* Fehlermedung und Rckgabe -1 */
	clear_screen();
	write_position(20,10,open_error,1);
	write_position(20,12,text1,1);
	printf(open_error);
	kbhit();
	return flag;
      }
    else
      {                                 /* Einlesen der Datenstruktur   */
	for(i=0;i<=7;i++)
	  {
	    fehler=fscanf(date_file,"%s",&wert);
	    date.s[i]=atof(wert);
	    if (fehler==-1)                 /* Fehler beim Schreiben ?      */
	      {                             /* Wenn ja, Meldung, Rckgabe -1 */
		clear_screen();
		write_position(20,10,read_error,1);
		write_position(20,12,text1,1);
		kbhit();
		flag=-1;
		break;
	      }
	    fehler=fscanf(date_file,"%s",&wert);
	    date.n[i]=atof(wert);
	    if (fehler==-1)                 /* Fehler beim Schreiben ?      */
	      {                             /* Wenn ja, Meldung, Rckgabe -1 */
		clear_screen();
		write_position(20,10,read_error,1);
		write_position(20,12,text1,1);
		kbhit();
		flag=-1;
		break;
	      }
	  }
	fclose(date_file);
      }
  return flag;
}

/************************************************************************/

int write_datefile(void)                /* Rckgabewert der Funktion    */
					/* 0=OK, -1=Fehler              */
{
  /* Bildschirmmeldungen */
  char write_error[]=" Fehler beim Schreiben der Abgleichdaten ! ";
  char open_error[]=" Fehler beim ffnen der Abgleichdatei ! ";
  char text1[]=" Beenden des Programms mit beliebiger Taste ";
  int i,fehler,flag;                    /* lokale Hilfsvariablen        */

  flag=0;                               /* Variable initialisieren      */
  date_file=fopen(file_name,"wb");      /* Datei zum Schreiben ffnen   */
  if (date_file==NULL)                  /* ffnen erfolgreich ?         */
      {                                 /* Wenn nicht, Meldung          */
	clear_screen();
	write_position(20,10,open_error,1); /* Rckgabe -1              */
	write_position(20,12,text1,1);
	printf(open_error);
	kbhit();
	flag=-1;
      }
    else                                /* Wenn erfolgreich             */
      {                                 /* Datenstrucktur schreiben     */
	for(i=0;i<=7;i++)
	  {
	    fehler=fprintf(date_file," %E %c%c",date.s[i],0x0D,0x0A);
	    if (fehler==-1)                 /* Fehler beim Schreiben ?      */
	      {                             /* Wenn ja, Meldung, Rckgabe -1 */
		clear_screen();
		write_position(20,10,write_error,1);
		write_position(20,12,text1,1);
		kbhit();
		flag=-1;
		break;
	      }
	    fehler=fprintf(date_file," %E %c%c",date.n[i],0x0D,0x0A);
	    if (fehler==-1)                 /* Fehler beim Schreiben ?      */
	      {                             /* Wenn ja, Meldung, Rckgabe -1 */
		clear_screen();
		write_position(20,10,write_error,1);
		write_position(20,12,text1,1);
		kbhit();
		flag=-1;
		break;
	      }
	  }
	fclose(date_file);               /* Datei schlieen              */
      }
  return flag;
}

/************************************************************************
 ***                Menusteuerung                                     ***
 ************************************************************************/

void output_menu(void)
{
  char text1[]="   Digitale Ausgnge    ";
  char text2[]=" F1 : Ausgang 1 =       ";
  char text3[]=" F2 : Ausgang 2 =       ";
  char text4[]="      Ende = F10        ";
  char low[]=" LOW  ";
  char high[]=" HIGH ";
  int i,a1,a2;
  int x=25;
  int y=12;

  clear_screen();                       /* Bildschirmdarstellung        */
  write_position(x,y-2,text1,1);
  write_position(x,y,text2,1);
  write_position(x,y+2,text3,1);
  write_position(x,y+4,text4,1);
  do                                    /* in Schleife verharren        */
    {                                   /* bis Taste F10 gedrckt       */
      a1=(output & 32);                 /* Ausgang 1 = Bit 6            */
      a2=(output & 16);                 /* Ausgang 2 = Bit 5            */
      if (a1!=32)
	  write_position(x+18,y,high,1);
	else
	  write_position(x+18,y,low,1);
      if (a2!=16)
	  write_position(x+18,y+2,high,1);
	else
	  write_position(x+18,y+2,low,1);
      i=kbhit();
      if (i==-1)                        /* Ist Taste gedrckt ?         */
	{                               /* Wenn ja, dann                */
	  key=getch();                  /* ASCII-Wert lesen             */
	  if (key==0)
	    {
	      key=getch();              /* Scan-Code holen              */
	      switch (key)
		{                       /* Ausgabe nur bei Tastendruck  */
		  case 59:{             /* Bei F1 : Ausgang 1           */
			    output=output ^ 32;
			    outportb(address,(channel | output));
			    a1=a1 ^ 32;
			    break;
			  }
		  case 60:{             /* Bei F2 : Ausgang 2            */
			    output=output ^ 16;
			    outportb(address,(channel | output));
			    a2=a2 ^ 16;
			    break;
			  }
		}
	    }
	}
    }
  while (key!=68);                /* Wenn F10, dann ende          */


}

/************************************************************************/

void input_menu(void)                   /* Unterprogramm fr digitale   */
{                                       /* Eingnge                     */
  char text1[]="  Digitale Eingnge  ";
  char text2[]=" Eingang 1   =       ";
  char text3[]=" Eingang 2   =       ";
  char text4[]=" Eingang 3   =       ";
  char text5[]="      Ende = F10     ";
  char low[]=" LOW  ";
  char high[]=" HIGH ";
  int i;
  int x=25;
  int y=12;

  clear_screen();                       /* Bildschirmdarstellung        */
  write_position(x,y-2,text1,1);
  write_position(x,y,text2,1);
  write_position(x,y+2,text3,1);
  write_position(x,y+4,text4,1);
  write_position(x,y+6,text5,1);
  do
    {
      i=inport(address+1);              /* Einlesen des Wertes          */
      if ((i & 32)==32)                 /* Eingang 1 = Bit 6            */
	  write_position(x+15,y,high,1);
	else
	  write_position(x+15,y,low,1);
      if ((i & 64)==64)                 /* Eingang 2 = Bit 7            */
	  write_position(x+15,y+2,high,1);
	else
	  write_position(x+15,y+2,low,1);
      if ((i & 128)==128)               /* Eingang 3 = Bit 8            */
	  write_position(x+15,y+4,high,1);
	else
	  write_position(x+15,y+4,low,1);

      if (kbhit()==-1)
	{
	  key=getch();
	  if (key==0)
	      key=getch();
	    else
	      key=0;
	}
    }
  while (key!=68);                      /* Ausgeben bis F10 gedrckt   */
}

/************************************************************************/

void calibrate(void)
{
  int i;                                /* lokale Hilfsvariable         */
  char chan[4];                         /* String fr Kanalnummer       */
  float wert;                           /* Variable fr Einlesen        */
  char key_input[20];                   /* String fr Tastatureingabe   */

  calibrate_menu();                     /* Menu darstellen              */
  key=0;                                /* Variable fr Taste initialisieren */
  do                                    /* in Schleife verharren        */
    {                                   /* bis Taste F1,F2 oder F10 gedrckt */
      chan[0]=32;
      chan[1]=channel+48;
      chan[2]=32;
      chan[3]='\0';
      write_position(53,12,"",1);
      textattr(128+64+14);
      cprintf(chan);
      i=kbhit();
      if (i==-1)                        /* Ist Taste gedrckt ?         */
	{                               /* Wenn ja, dann                */
	  key=getch();                  /* ASCII-Wert lesen             */
	  if ((key<=55) && (key>=48))
	    {
	      channel=key-48;
	    }
	  if (key==0)
	    {
	      key=getch();              /* Scan-Code holen              */
	      switch (key)
		{
		  case 59:{             /* Bei F1 : Nullabgleich        */
			    write_position(20,20,"  Widerstand 1 anschliessen und Wert eingeben  ",1);
			    write_position(20,22,"           Mit RETURN besttigen               ",1);
			    write_position(20,21," Wert :                                        ",1);
			    write_position(28,21,"",1);
			    _setcursortype(_SOLIDCURSOR);
			    scanf("%s",&key_input);
			    _setcursortype(_NOCURSOR);
			    /* Komma und Dezimalpunkt erlaubt */
			    for (i=0;i<=20;i++)
			      {
				if (key_input[i]==',')
				  key_input[i]='.';
				if (key_input[i]=='\0')
				  break;
			      }
			    wert=atof(key_input);
			    write_position(20,20,"                                                      ",0);
			    write_position(20,21,"                                                      ",0);
			    write_position(20,22,"                                                      ",0);
			    date.y1[channel]=wert;
			    date.x1[channel]=read_board(channel);
			    break;
			  }
		  case 60:{             /* Bei F2 : Endabgleich         */

			    write_position(20,20,"  Widerstand 2 anschliessen und Wert eingeben  ",1);
			    write_position(20,22,"           Mit RETURN besttigen               ",1);
			    write_position(20,21," Wert :                                        ",1);
			    write_position(28,21,"",0);
			    _setcursortype(_SOLIDCURSOR);
			    scanf("%s",&key_input);
			    _setcursortype(_NOCURSOR);
			    /* Komma und Dezimalpunkt erlaubt */
			    for (i=0;i<=20;i++)
			      {
				if (key_input[i]==',')
				  key_input[i]='.';
				if (key_input[i]=='\0')
				  break;
			      }
			    wert=atof(key_input);
			    write_position(20,20,"                                                      ",0);
			    write_position(20,21,"                                                      ",0);
			    write_position(20,22,"                                                      ",0);
			    date.y2[channel]=wert;
			    date.x2[channel]=read_board(channel);
			    break;
			  }
		}
	    }
	  else
	    key=1;
	}
    }
  while (key!=68);                      /* Wenn F10, dann ende          */
  for (i=0;i<=7;i++)
    {
      if (!((date.y1[i]==0) && (date.y2[i]==0) && (date.x1[i]==0) && (date.x2[i]==0)))
      {
      if ((date.y1[i]==date.y2[i]) || (date.x1[i]==date.x2[i]))
	  {                             /* Wenn ja, Fehlermeldung       */
	    clear_screen();
	    write_position(20,12," Abgleichfehler bei Kanal    ",1);
	    chan[1]=i+48;
	    write_position(46,12,chan,1);
	    write_position(20,13," Weiter mit beliebiger Taste ",1);
	    getche();                   /* Auf Tastendruck warten       */
	  }
	else
	  {                             /* Berechnung der Geraden       */
	    date.s[i]=((date.y2[i]-date.y1[i])/(date.x2[i]-date.x1[i]));
	    date.n[i]=(date.y1[i]-date.s[i]*date.x1[i]);
	  }
	}
    }
}

/************************************************************************/

void show(void)
{
  int i;                              /* lokale Hilfsvariablen        */
  double voltage,digital,wert;

  /* Koeffizienten fr Ausgleichsfunktion des PT100 */
  double a3=1.0825E-6,a2=6.14993E-4,a1=2.40341,a0=-247.545;

  show_menu();                          /* Menu darstellen             */
  i=0;                                  /* Die Laufvariable wird nur hier
					   initialisiert um auf einen
					   Tastendruck whrend der
					   Ausgabeschleife zu reagieren */
  key=0;                                /* Variable fr Taste initialisieren */
  do                                    /* in Schleife verharren        */
    {                                   /* bis Taste F10 gedrckt       */
      for (;i<=7;i++)                   /* Alle Kanle einlesen         */
	{
	  digital=read_board(i);        /* Spannungswert Kanal i holen, */
	  voltage=date.s[i]*digital+date.n[i]; /* berechnen             */
	  write_position(42,i+10,"",0);  /* und ausgeben                 */
	  wert=(((a3*voltage)+a2)*voltage+a1)*voltage+a0;
	  printf(" %6.2f ",wert);
	  if (kbhit()==-1)              /* Ist Taste gedrckt ?         */
	    {                           /* Wenn ja, dann                */
	      key=getch();              /* ASCII-Wert lesen             */
	      if (key==0)
		key=getch();            /* Scan-Code holen              */
	       else
		key=0;
	      break;
	    }
	}
      if (i==8)                         /* Laufvariable reinitialisieren */
	i=0;
    }
  while (key!=68);                      /* Wenn F10, dann ende          */

}

/************************************************************************
 ***                Unterprogramme zur Menusteuerung                  ***
 ************************************************************************/

void main_menu(void)
{
  char text1[]=" Temperaturen anzeigen = F1  ";
  char text2[]=" Meeingnge eichen    = F2  ";
  char text3[]=" Digitalausgnge       = F3  ";
  char text4[]=" Digitaleingnge       = F4  ";
  char text5[]=" Messung beenden       = F10 ";
  int  x=25;
  int  y=10;

  clear_screen();
  write_position(x,y  ,text1,1);
  write_position(x,y+2,text2,1);
  write_position(x,y+4,text3,1);
  write_position(x,y+6,text4,1);
  write_position(x,y+8,text5,1);
}

/************************************************************************/

void calibrate_menu(void)
{
  char info1[]=" Vor einer Inbetriebnahme empfiehlt sich ein Ablgeich der Meeingnge.";
  char info2[]="           Dadurch wird die Genauigkeit der Meergebnisse erhht.     ";
  char text0[]=" >>>>>> Mestelle whlen (0 - 7)     <<<<<<";
  char text1[]=" Nullabgleich          = F1  ";
  char text2[]=" Endwertabgleich       = F2  ";
  char text3[]=" Messung beenden       = F10 ";
  int  x=25;
  int  y=14;

  clear_screen();
  write_position(x-19,y-5,info1,1);
  write_position(x-19,y-4,info2,1);
  write_position(x-5,y-2,text0,1);
  write_position(x,y  ,text1,1);
  write_position(x,y+2,text2,1);
  write_position(x,y+4,text3,1);
}

/************************************************************************/

void show_menu(void)
{
  char text1[]="     Anzeige der Temperaturen  ";
  char text2[]="          Ende = F10           ";
  char text3[]=" Meeingang   =             C ";
  int  x=25;
  int  y=10;
  int  i;
  char kanal[2];

  clear_screen();
  write_position(x,y-2,text1,1);
  write_position(x,y+9,text2,1);
  for (i=0;i<=7;i++)
    {
      write_position(x,y+i,text3,1);
      kanal[0]=i+48;
      kanal[1]='\0';
      write_position(x+12,y+i,kanal,1);
    }
}

/************************************************************************
 ***                Unterprogramme zur Bildschirmsteuerung            ***
 ************************************************************************/

void clear_screen(void)
{
  /* Diese Funktion dient zum lschen des Bildschirmes, sowie dessen
     Einstellung. Der Hintergrund wird auf die Farbe Blau eingestellt.
     Es wird des weiteren ein Fenster mit der Firmenadresse dargestellt.  */

  char rahm1[]="ͻ";
  char rahm2[]="                                                         ";
  char rahm3[]="ͼ";
  char text1[]=" ERMA-Electronic GmbH, Max-Eyth-Str. 8, 7717 Immendingen ";
  char text2[]="      Tel. 07462/7381               Fax 07462/7554       ";
  char text3[]="";

  textbackground(BLUE);                 /* Hintergrundfarbe schwarz     */
  textcolor(CYAN);                      /* Vordergrundfarbe hellgrau    */
  clrscr();                             /* Bildschirm lschen           */

  /* Erzeugen eines Fensters */
  gotoxy(12,2);
  textattr(0x7F);
  cprintf(rahm1);
  gotoxy(12,3);
  cprintf(rahm2);
  gotoxy(12,4);
  cprintf(rahm2);
  gotoxy(12,5);
  cprintf(rahm3);
  /* Erzeugen des Fenstertextes */
  textattr(0x70);
  gotoxy(13,3);
  cprintf(text1);
  gotoxy(13,4);
  cprintf(text2);
  /* Erzeugen des Fensterschattens */
  gotoxy(13,6);
  cprintf(text3);
  gotoxy(71,3);
  cprintf("");
  gotoxy(71,4);
  cprintf("");
  gotoxy(71,5);
  cprintf("");
  textattr(0x03);
}

/***************************************************************************/

void input_win(char *text)
{
  /* Diese Funktion erzeugt ein Fenster fr Eingaben. Die Eingabe erfolgt
     jedoch auerhalb dieser Funktion. Das Fenster kann mit einem einzeiligen
     Text versehen werden. Sollte dieser Text zu lang sein, so wird ber das
     Fenster hinausgeschrieben.                                           */

  char rahm1[]="ͻ";
  char rahm2[]="                                                           ";
  char rahm3[]="ͼ";
  char rahm4[]="";
  char rahm5[]="                                                          ";
  int x=12;                                 /* Position des Fensters      */
  int y=13;

  /* Erzeugen eines Fensters */
  gotoxy(x,y);
  textattr(0x7F);
  cprintf(rahm1);
  gotoxy(x,y+1);
  cprintf(rahm2);
  gotoxy(x,y+2);
  cprintf(rahm2);
  gotoxy(x,y+3);
  cprintf(rahm3);
  /* Erzeugen des Fenstertextes */
  textattr(0x70);
  gotoxy(x+2,y+1);
  cprintf(text);
  textattr(0x7E);
  gotoxy(x+2,y+2);
  cprintf(rahm5);
  /* Erzeugen des Fensterschattens */
  textattr(0x70);
  gotoxy(x+1,y+4);
  cprintf(rahm4);
  gotoxy(x+61,y+1);
  cprintf("");
  gotoxy(x+61,y+2);
  cprintf("");
  gotoxy(x+61,y+3);
  cprintf("");
  textattr(0x7E);
  gotoxy(x+2,y+2);

}

/************************************************************************/

void error_win(char *text1,char *text2)
{
  /* Diese Funktion erzeugt ein Fenster fr Fehlermeldungen. Die Meldung
     besteht aus zwei Zeilen. Die Lnge dieser Zeilen wird nicht
     berprft, sodas bei zu langen Zeilen ber das Fenster hinaus-
     geschrieben wird. Die Funktion wird erst verlassen, wenn eine Taste
     gedrckt wird.                                                       */

  char rahm1[]=" Fehler ͻ";
  char rahm2[]="                                                  ";
  char rahm3[]="ͼ";
  char rahm4[]="";
  char text3[]="      Bitte drcken Sie eine beliebige Taste      ";
  int x=15;                                 /* Position des Fensters      */
  int y=13;

  /* Erzeugen eines Fensters */
  gotoxy(x,y);
  textattr(0x0F4);                          /* Rahmen rot und blinkend    */
  cprintf(rahm1);
  gotoxy(x,y+1);
  cprintf(rahm2);
  gotoxy(x,y+2);
  cprintf(rahm2);
  gotoxy(x,y+3);
  cprintf(rahm2);
  gotoxy(x,y+4);
  cprintf(rahm3);
  /* Erzeugen des Fenstertextes */
  textattr(0x70);
  gotoxy(x+1,y+1);
  cprintf(text1);
  gotoxy(x+1,y+2);
  cprintf(text2);
  textattr(0x13);
  gotoxy(x+1,y+3);
  cprintf(text3);
  textattr(0x70);
  /* Erzeugen des Fensterschattens */
  gotoxy(x+1,y+5);
  cprintf(rahm4);
  gotoxy(x+52,y+1);
  cprintf("");
  gotoxy(x+52,y+2);
  cprintf("");
  gotoxy(x+52,y+3);
  cprintf("");
  gotoxy(x+52,y+4);
  cprintf("");
  textattr(0x03);
  do {} while (!kbhit());
  x=getch();
  if (x==0)
    x=getch();
}

/************************************************************************/

void write_position(int x,int y,char *text,int attr)
{
  if (attr==0)
    textattr(0x10);
  if (attr==1)
    textattr(0x3E);
  gotoxy(x,y);
  cprintf(text);

}


